/*
///////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: error_gen.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:23
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     This module is responsible for generating the error signal lderr_n
     and the contents of the 4 LSB of the control words.

     inj_err_led indicates that the error push button has been pressed
     and will remain on until the block containing the error has been
     fully transferred. This LED is only useful for the lowest frequencies
     of LCLK.

     The signals from the switches SW5, SW6, SW7 and SW10 are called
     block_basis, data_err, ctrl_err and inj_err, respectively.

///////////////////////////////////////////////////////////////////////////////
*/


module error_gen (lclk, sysenable, reset, enable, inj_err, start,
                  int_ctrl, ctrl_err, data_err, block_basis,
		  int_lderr, ld[3:0], inj_err_led);
		  
  input lclk, sysenable, reset, enable, inj_err, start, int_ctrl;
  input ctrl_err, data_err, block_basis;
  output int_lderr, inj_err_led;
  output [3:0] ld;

  reg int_lderr;
  reg [3:0] ld;

  reg pending_error, generate_error;
  reg [2:0] counter;


  wire inj_err_led = ~(pending_error | generate_error);  // Must be inverted
                                                         // to drive the LED
                                                         // correctly.

  
//======================================================================
//  pending_error is a flag used to memorise that the 'error' button has
//  been pressed. This flag is used to set a second flag (generate_error)
//  that will be active during the entire next-coming data block.
//
//  As soon as the new block starts to be generated ('start' == 1)
//  the value of pending_error is passed to generate_error.
//  At this stage, pending_error waits for the next pulse of inj_err
//  (indication that 'error' has been pressed).
//
//  'sysenable' does not have the top priority in order to allow inj_err
//  to be latched at any time, even if UXOFF# is active (this fixes
//  bug 2 of Slidas 1.0)
//======================================================================
  
  always @(posedge lclk)
    if (sysenable & reset) pending_error <= #1 0;
    else
      if ((sysenable & start) | ~pending_error)
	pending_error <= #1 inj_err;


//======================================================================
//  Generation of the flag generate_error.
//  This flag is used as an enable signal for the generation of errors
//  on ld[3:0] and lderr_n.
//  It is set by pending_error together with sysenable and start, and 
//  it is reset when a control word is sent (int_ctrl == 1).
//======================================================================      

  always @(posedge lclk)
    if (sysenable)
      if (reset) generate_error <= #1 0;
      	else if (start) generate_error <= #1 pending_error;
	  else if (enable & int_ctrl) generate_error <= #1 0;



//======================================================================
//  Generation of 'int_lderr'
//  counter[1:0]: value of 3 indicates fourth data word.
//  data_err: switch controlling the generation of errors in data words.
//  ctrl_err: switch controlling the generation of errors in control words.
//  block_basis: switch selecting block basis or word-by-word error det.
//  int_ctrl: indicates that a control word is being sent.
//======================================================================
	  
  always @(posedge lclk)
    if (sysenable)
      if (reset) int_lderr <= #1 0;
      else if (enable)
	int_lderr <= #1 (generate_error &&
	          ( ((counter[1:0] == 3) && data_err && ~block_basis) ||
     		    (int_ctrl & (ctrl_err | data_err))) )?1:0;


		    
//======================================================================
//  Generation of ld[3:0]
//  (usage of counter[2]: see explanation in the next comment.)
//======================================================================

  always @(posedge lclk)
    if (sysenable)
      if (enable)
      	ld[3:0] <= #1 {2'b0, generate_error & ctrl_err,
                             generate_error & data_err & counter[2]};



//======================================================================
//  Generation of the counter.
//
//  The counter is used to generate data errors on the correct place,
//  that is, on the fourth data word of a block.
//  It is reset when a new block starts, and counts until it
//  reaches the value 4, where it remains.
//
//  Letting the counter remain in that value allows two things: first,
//  the value 3 (which indicates the position of the word with an error)
//  is only seen once in a block with 4 or more data words, thus
//  generating only one error in a data block. Second,
//  the bit counter[2] indicates, when 1, that four or more words have
//  been sent. This is used to provide the correct error information
//  in ld[3:0], since otherwise blocks with length 0 and 1 would have
//  control words indicating errors in data words which didn't exist.
//======================================================================

  always @(posedge lclk)
    if (sysenable)
      if (start) counter <= #1 0;
	else if (enable) counter <= #1 counter[2]?counter:counter + 1;

endmodule



