/* 
   Project: Altera circuit of SLIDAS 

   File name: internal_trigger.v
   Version:   1.3
   Updated:   3/8/99 

   Author: Gillian Damerell, ECP division, CERN


   
-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
      1.3    |  GD      |  3/8/99     | This is a new module which generates
             |          | 	      | the internal trigger, at frequencies
	     |		| 	      | of 50 - 150 kHz.
	     |		|	      | 
    1.4	     |	GD	|  24/8/99    | No `ifdef exemplar statement.
	     |		|	      |
-------------------------------------------------------------------------------



     Module description:
     -------------------
This generates the internal trigger from clk66, a 40MHz clock.  To go from 
40MHz to, for example, 100kHz, is the equivalent of dividing by 400.  If the 
duty_switch is set to 100kHz, then the trigger_counter will be loaded with the 
value 399.  Every time there is a clock pulse, it will count down by 1, and 
when it reaches 0 an internal trigger pulse will be sent.  The process will 
then repeat.
-------------------------------------------------------------------------------
*/

module internal_trigger (clk66, duty_switch, sysreset, ureset, 
			internal_trigger);


  input clk66, sysreset, ureset;
  input [3:0] duty_switch;
  output internal_trigger;

  reg internal_trigger;
  reg [10:0] trigger_counter, trigger_length;

  reg [3:0] sync_duty_switch;    //duty_switch synchronised to clk66
  
  wire ureset_s;                 //ureset level synchronised to clk66

  wire sysreset_s;		 //sysreset level synchronised to clk66

  parameter
    RUN          = 0,
    SINGLE_STEP  = 1,
    SINGLE_BLOCK = 2,
    _50kHz       = 3,
    _75kHz       = 4,
    _90kHz       = 5,
    _100kHz      = 6,
    _110kHz      = 7,
    _125kHz      = 8,
    _150kHz      = 9;



//-----------------------------------------------------------------------------
// Synchronisation of duty_switch, ureset and sysreset to clk66.

  always @(posedge clk66) sync_duty_switch <= #1 duty_switch;

  sync_level  ureset_sync(clk66, ureset_s, ureset);

  sync_level  sysreset_sync(clk66, sysreset_s, sysreset);


//----------------------------------------------------------------------------- 
// trigger_length is the value with which the trigger counter will be loaded
// for the count down.

  always @(posedge clk66)
    case (sync_duty_switch)
      _50kHz  : trigger_length <= #1 (40000/50 - 1);    // 799
      _75kHz  : trigger_length <= #1 (40000/75 - 1);    // 532
      _90kHz  : trigger_length <= #1 (40000/90 - 1);    // 443
      _100kHz : trigger_length <= #1 (40000/100 - 1);   // 399
      _110kHz : trigger_length <= #1 (40000/110 - 1);   // 363
      _125kHz : trigger_length <= #1 (40000/125 - 1);   // 319
      _150kHz : trigger_length <= #1 (40000/150 - 1);   // 266
      default : trigger_length <= #1 0;
    endcase


    
//----------------------------------------------------------------------------- 
// trigger_counter is used to determine the pace of the internal trigger.  It
// is constantly counting, and every time it reaches 0, the internal_trigger 
// is pulsed. 

  always @(posedge clk66)
    if (sysreset_s || ureset_s || (trigger_counter == 0))
      trigger_counter <= #1 trigger_length;
    
    else trigger_counter <= #1 trigger_counter -1;
    
    
  always @(posedge clk66)
    internal_trigger <= #1 (trigger_counter == 0)?1:0;
    

//-----------------------------------------------------------------------------

endmodule
    

/*
`ifdef exemplar

module sync_level(lclk, out, in);
  input lclk, in;
  output out;
endmodule


`else
 
`include "sync_level.v"   
    
`endif    
*/
