/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: length_controller.v
//   Version:   
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
/////////////////////////////////////////////////////////////////////////////

-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
     3.1     |   PBY    | 05-04-1997  | - See SCCS comments for this file
             |          |             |   ('sccs prt length_controller.v')
             |          |             |
     3.2     |   PBY    | 08-04-1997  | - Changed @(block_length_pointer) to
             |          |             |   be @(posedge lclk) in the block
             |          |             |   where 'offset' is generated. This
             |          |             |   introduces flip-flops instead of
             |          |             |   only logic (again, for timing
             |          |             |   improvement after compilation).
             |          |             |
     3.3     |    GD    | 06-07-1999  | - put this bit inside comment marks  
             |          |             |
     4.2     |   GD	| 12/7/99     | - altered the lengths to be in line
	     |		|	      |   with the new data format.  Also 
	     |		|	      |   changed what happens when the length
	     |		|	      |   alternates - see comment below.
	     |		|	      |   Changed the flag length_is_1, which 
	     |		|	      |   no longer applies, for the flag
	     |		|	      |   length_minus_3, ie no status words.
	     |		|	      |
     4.3     |    GD	| 13/7/99     | made the normal_block_length different
	     |		|	      | depending on the pattern_switch.
	     |		|	      |
-------------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     LENGTH_CONTROLLER
     =================
     This module controls the amount of data words generated by the slidas.
     The number of data words depends mainly on the 'block_length' rotary
     switch, but also on the switch 'pattern_type' and the pins UDW.

     The length is normally defined by the switch 'block_length'. The
     exceptions are the following modes:

     TEST MODE
     =========
     The block length depends on the lines UDW, that is, on the width of
     the generated words. For 8-bit words, the block length is 8. For
     16-bit words the block length is 16 and for 32-bit words the block
     length is 32.

     ROB_alternating_lengths
     =======================
     In positions 2 and up of the length rotary switch, the block length
     changes after a block has been sent. The signal
     new_block triggers the change in length. The lengths will cycle as
     follows:
   
     (length + 25%) -> length -> (length - 25%) -> length

     where 'length' is indicated by the length rotary switch.

     Upon a reset the block length in this mode is set to (length + 25%).

     In positions 0,1 of length_switch, the length is the one indicated
     by the switch (the length doesn't change from on block to another).

/////////////////////////////////////////////////////////////////////////////
*/



`define  test_mode        0
`define  walking_1        1
`define  walking_0        2
`define  FFFF_0000        3
`define  AAAA_5555        4
`define  random_data      5
`define  random_no_ctrl   6
`define  random_all_ctrl  7
`define  ROB_sel_lenghts  8
`define  ROB_4_lengths    9

`define    minus_3   0
`define    _0_       1
`define    _48_      2
`define    _112_     3
`define    _240_     4
`define    _496_     5
`define    _1_K_     6
`define    _2_K_     7
`define    _4_K_     8
`define    _8_K_     9

`define    _8_bits   2
`define    _16_bits  1
`define    _32_bits  0




module length_controller(lclk, sysenable, length_reset, new_block,
                         length_is_0, length_minus_3,
			 pattern_switch, length_switch, udw,
		         block_length);
		      

  input lclk, sysenable;
  input length_reset;             // initialises the block_length value,
                                  // which is exported to the other modules.
                                  // This signal has a meaning only on
                                  // pattern_switch position 9, where the
                                  // block length is changing from one block
                                  // to another
  
  input new_block;                // makes block_length to change value (in
                                  // a 4-position cycle). See offset.
  
  input [3:0] pattern_switch;     // Rotary swich after synchronisation
  input [3:0] length_switch;      // Rotary swich after synchronisation
  input [1:0] udw;                // UDW, directly from outside.
  output length_is_0;		  // signal to tell the state machine
                                  // that 0 words are to be sent.
                                  // This is necessary because the state
                                  // machine's overhead for starting the word
                                  // counter and checking for the end_of_block
                                  // signal (generated by word_counter) is
                                  // too big for such short data blocks.
  
  output length_minus_3;	  // signal to tell the state machine that
				  // no status words are to be included.


  output [15:0] block_length;     // This is the value that is seen from the
                                  // other modules. It is used by data_gen
                                  // to send it as a normal data word, and
                                  // by word_counter, as the value to load
                                  // and start counting down from there.
//----------------------------------
  
  reg length_is_0, length_minus_3;   // See above
  

  reg [15:0] block_length;        // See above

  
  reg [15:0] ROB_block_length;    // block_length value to use with
                                  // pattern_switch on position 9.
  
  reg [15:0] test_block_length;   // block_length value to use with
                                  // pattern_switch on position 0 (test mode).
  
  reg [15:0] normal_block_length; // block_length value to use with
                                  // pattern_switch on any other position.
  

  integer offset;                 // used in combination with normal_block_
                                  // length to obtain ROB_block_length.
                                  // With this value ROB_block_length will
                                  // cycle through 4 values, using normal_
                                  // block_length as the base.

  reg [15:0] modified_block_length; // the sum of normal_block_length and
                                    // offset
  
  reg [1:0] block_length_pointer; // block_length_pointer selects which
                                  // offset to use (used during ROB mode
                                  // with varying lengths). It is just a
                                  // counter cycling through 0,1,2,3.
                                  // Reset by length_reset.

	     
  always @(posedge lclk)
    if (sysenable)
    if (length_reset) block_length_pointer <= #1 0;
    else if (new_block) block_length_pointer <= #1 block_length_pointer + 1;

      

// the offset is 25% of the total length, which is the data length plus 16

  always @(posedge lclk)
    offset <= #1 (normal_block_length + 16)/4;

/*  If the pattern switch is in position 9, then you cycle through the normal length plus 25% of the total length, the normal length, the normal length minus 25% of the total length, the normal length, etc.  This is controlled here by the block_length_pointer, which cycles through 0, 1, 2 and 3.  */

  always @(posedge lclk)
    case (block_length_pointer)
      0: modified_block_length <= #1 normal_block_length + offset;
      1: modified_block_length <= #1 normal_block_length;
      2: modified_block_length <= #1 normal_block_length - offset;
      3: modified_block_length <= #1 normal_block_length;
    endcase


/* If you want to change the value of any of the positions, or any of the lengths, make sure that they will not overflow or underflow the value of ROB_block_length.  For example, at the moment if the length_switch is in positions 0 or 1, the block length is just set to the normal_block_length, ie it does not cycle through.  This is because there are no data words in positions 0 or 1, so if the total length is reduced by 25% then the no. of data words would become negative, which is not possible.  */

// obtaining the value for ROB_block_length
  always @(posedge lclk)
    case (length_switch)
      0,1: ROB_block_length <= #1 normal_block_length;
      default:   ROB_block_length <= #1 modified_block_length;
    endcase



  always @(posedge lclk)                           // test_block_length depends
    case (udw)                                     // only on UDW, and not
      `_8_bits:  test_block_length <= #1 16'd 8;   // on length_switch
      `_16_bits: test_block_length <= #1 16'd 16;
      `_32_bits: test_block_length <= #1 16'd 32;
      default:   test_block_length <= #1 16'd 32;
    endcase
    
/*  If the pattern switch is in position 8 or 9, then the data length, which is 
defined here, should be the total length minus the length of the header and 
trailer used in ROB blocks.  If it is in positions 6 or 7, then the data length 
should be the total length, since there are no words other than those counted 
by the "normal_block_length". Otherwise, the data length should be equal to the 
total length minus 2 , since there are 2 control words used to mark the start 
and finish of blocks. */
    
  always @(posedge lclk)
    case (pattern_switch)
 
      8,9:     case (length_switch)                          
                 `minus_3: normal_block_length <= #1 16'd 0;  
                 `_0_:     normal_block_length <= #1 16'd 0;  
                 `_48_:    normal_block_length <= #1 16'd 48; 
      	         `_112_:   normal_block_length <= #1 16'd 112;
                 `_240_:   normal_block_length <= #1 16'd 240;
                 `_496_:   normal_block_length <= #1 16'd 496;
                 `_1_K_:   normal_block_length <= #1 16'd 1008;
                 `_2_K_:   normal_block_length <= #1 16'd 2032;
                 `_4_K_:   normal_block_length <= #1 16'd 4080;
                 `_8_K_:   normal_block_length <= #1 16'd 8176;
                 default:  normal_block_length <= #1 16'd 0;
               endcase

      6,7:     case (length_switch)                          
                 `minus_3: normal_block_length <= #1 16'd 13;  
                 `_0_:     normal_block_length <= #1 16'd 16;  
                 `_48_:    normal_block_length <= #1 16'd 64; 
      	         `_112_:   normal_block_length <= #1 16'd 128;
                 `_240_:   normal_block_length <= #1 16'd 256;
                 `_496_:   normal_block_length <= #1 16'd 512;
                 `_1_K_:   normal_block_length <= #1 16'd 1024;
                 `_2_K_:   normal_block_length <= #1 16'd 2048;
                 `_4_K_:   normal_block_length <= #1 16'd 4096;
                 `_8_K_:   normal_block_length <= #1 16'd 8192;
                 default:  normal_block_length <= #1 16'd 13;
               endcase

      default: case (length_switch)                          
                 `minus_3: normal_block_length <= #1 16'd 11;  
                 `_0_:     normal_block_length <= #1 16'd 14;  
                 `_48_:    normal_block_length <= #1 16'd 62; 
      	         `_112_:   normal_block_length <= #1 16'd 126;
                 `_240_:   normal_block_length <= #1 16'd 254;
                 `_496_:   normal_block_length <= #1 16'd 510;
                 `_1_K_:   normal_block_length <= #1 16'd 1022;
                 `_2_K_:   normal_block_length <= #1 16'd 2046;
                 `_4_K_:   normal_block_length <= #1 16'd 4094;
                 `_8_K_:   normal_block_length <= #1 16'd 8190;
                 default:  normal_block_length <= #1 16'd 11;
               endcase
    endcase


   
  always @(posedge lclk)                               // Only one of the
    casex (pattern_switch)                             // xxx_block_length 
      4'h0:    block_length <= #1 test_block_length;   // values is applicable
      4'h9:    block_length <= #1 ROB_block_length;    // at any time. Here
      default: block_length <= #1 normal_block_length; // it is selected
    endcase
  
    
  always @(posedge lclk) begin                            // length_is_0 can
    length_is_0 <= #1 ((pattern_switch != `test_mode)&&   // and must only be
           	       (length_switch == `_0_))?1:0;      // asserted when not
		                                          // in test mode. The
    length_minus_3 <= #1 ((pattern_switch != `test_mode)&& // same applies to
           	    (length_switch == `minus_3))?1:0;      // length_minus_3.
  end		       
endmodule
