/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: lwen_pipe.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:29
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     This pipeline transports the value that becomes lwen# at the output pin
     of the Altera. At the input there is the value int_lwen, generated by
     state_machine.

     The signal pipe_reset resets the whole pipeline at once, on the
     clock edge, if sysenable is active.

     The output, lwen_n, behaves in a particular manner: sysenable does not
     'hold' the value at the output, as one might expect, but deasserts the
     signal. That is, if sysenable is active then lwen_n takes the value
     that is flowing through the pipeline, but if sysenable is low lwen_n
     is set to 1.

     This is the obvious way to behave for lwen_n. If uxoff# is active (low),
     the Altera freezes, but lwen_n must go high so that the 'frozen' word
     is not constantly transferred by a 'frozen' lwen_n.

/////////////////////////////////////////////////////////////////////////////
*/


module lwen_pipe (lclk, sysenable, pipe_reset, int_lwen, lwen_n);
  input lclk, sysenable, pipe_reset, int_lwen;
  output lwen_n;


  reg lwen2, lwen3, lwen4;
  reg int_lwen_n;
  wire lwen_n;


  always @(posedge lclk)
    if (sysenable)
      if (pipe_reset)
	begin
	  lwen2 <= #1 0;
	  lwen3 <= #1 0;
	  lwen4 <= #1 0;
	end
      else
     	begin
	  lwen2 <= #1 int_lwen;
	  lwen3 <= #1 lwen2;
	  lwen4 <= #1 lwen3;
	end

//-----------------------------------------------------------------------------
//  The following block implements a logic block, where int_lwen_n is
//  generated as to be the input to the output register. This way, the
//  logic is implemented inside the altera and the final register is placed
//  in a I/O register (to improve clock to output timing)
//-----------------------------------------------------------------------------
	
  always @(sysenable or pipe_reset or lwen4)
    if (sysenable)
      if (pipe_reset) int_lwen_n <= #1 1;
	else int_lwen_n <= #1 ~lwen4;
    else int_lwen_n <= #1 1;


//  output register, to be placed in the I/O cell.
//  always @(posedge lclk) lwen_n <= #1 int_lwen_n;

  ff_w_enable ff_lwen (lclk, lwen_n, int_lwen_n, 1'b1);
  // enable is always on.

endmodule


/*
`ifdef exemplar
    // if during compilation with Galileo

module ff_w_enable (lclk, out, in, enable);
  input lclk, in, enable;
  output out;
endmodule

`else
// this line should be uncommented if you want to do simulation at a lower
// level than altera.v
// `include "ff_w_enable.v"
`endif
*/

