/*
///////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS 
//
//   File name: data_gen4.v
//   Version:   1.1
//   Updated:   30/8/99
//
//   Author: Gillian Damerell
//
///////////////////////////////////////////////////////////////////////////////

-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
    1.1	     |	EB	| 31/8/99     | now called data_gen4.v, and implements 
    	     |		|  	      | the SCHWICK data format. based on
             |          |             | data_gen2.v version 1.1. 
	     |		|	      | Only include statements changed
     	     |  	|  	      |  
	     |		|	      | 
	     |		|	      |
-------------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     This module implements the data generator module of the SLIDAS.
     External signals select the kind of data that is to be generated and
     how. For example, apart from selecting walking_1 as type of data,
     the sequence must be reset at some time, as well as disabled when
     necessary.

     Signals and buses
     -----------------
     The bus 'selection' carries information about the type of data
     to be selected and which inputs of the multiplexers to select in order
     to guide the data generator outputs down to the output registers.

     In the case of the module walking_and_inverting, 'selection' is not
     enough to generate the appropriate sequence, so the bus 'pattern_switch'
     is also provided.

     There are also individual reset and enable lines for most of the
     data generator modules, as well as some other signals specific to
     individual modules.

     Although it is a bus, block_length is one of the data sources.

     Structure of data_gen
     ---------------------
     This module is structured as a 4-level pipelined multiplexer, with data
     generator sub-blocks or other data sources at the first level
     and the output registers for the lines ld, lwen_n, lctrl_n and lderr_n
     at the fourth level. Levels 2 and 3 are occupied by multiplexers.

     Pipelines
     ---------
     The pipelines for lwen_n, lderr_n and lctrl_n are provided just to
     introduce the correct amount of clock cycles with respect of the
     data word they belong to and that is flowing through the multiplexers.
     selection, however, is put into a pipeline so that it can select
     the right data source or multiplexer input at the right time. The
     bus is called selection2_ during the second clock cycle and selection3_
     during the third clock cycle.

     During the first clock cycle, bits 3 to 0 are only used as address lines
     by look_up_table.

     During the second clock cycle, bits 7 and 6 of selection2_ are used by
     one multiplexer and bits 5 and 4 by the other multiplexer. One could
     argue that only two bits would be necessary as selectors. The reason
     for having separate selectors is that for control words the output
     of both multiplexers are used. This is explained in the following
     paragraph.

     During the third clock cycle, bits 9 and 8 of selection3_ are used by
     the third level multiplexer to choose a data source or combine it
     with the information from error_gen, in the case of a control word.
     This multiplexer is 'trimmed' in the sense that not only one of
     the two inputs can be selected (selection3_[9:8] == 10 or 01), but
     they can also be combined by putting together 28 bits of one word
     and 4 bits of the other (selection3_[9:8] == 11). This feature is used
     to generate the control words, where the four lowest bits carry
     special information.


///////////////////////////////////////////////////////////////////////////////
*/


module data_gen (lclk, sysenable, pattern_switch, udw,
		 data_err, ctrl_err, block_basis, inj_err, inj_err_led, femb,
		 pattern0_jumper_on,
  
                 walking_and_inverting_reset, random_reset,
		 block_counter_reset, trigger_type_reset, data_counter_reset,
		 error_gen_reset, pipe_reset,
		 
                 walking_and_inverting_enable, random_enable,
		 block_counter_enable, trigger_type_enable,
		 data_counter_enable, error_gen_enable,

		 error_gen_start,

		 selection, int_lwen,
		 
		 block_length,

		 lderr_n, lwen_n, lctrl_n, ld);



  input          lclk, sysenable;
	       
  input          walking_and_inverting_reset, random_reset,
		 block_counter_reset, trigger_type_reset, data_counter_reset,
		 error_gen_reset, pipe_reset,

                 walking_and_inverting_enable, random_enable,
		 block_counter_enable, trigger_type_enable,
		 data_counter_enable, error_gen_enable;

  input[15:0]	 block_length;

  input[3:0]     pattern_switch;

  input[9:0]     selection;
  
  input[1:0]     udw;

  input          int_lwen, error_gen_start;

  input          data_err, ctrl_err, block_basis, inj_err, femb,
                 pattern0_jumper_on;

  output         lderr_n, lwen_n, lctrl_n, inj_err_led;
  
  output [31:0]  ld;

  wire [31:0]    walk_and_invert_q, random_q, look_up_table_q,
                 block_counter_q, trigger_type_q, data_counter_q,
	      	 big_muxer0_q, big_muxer1_q, mux_merger_q;
	      
  wire [3:0]     error_gen_q;


// int_lctrl is a flag that indicates that a control word is being selected.
  wire int_lctrl = (selection[9:8] == 2'b11)?1:0;

  wire [9:0]     selection2_, selection3_;

  
// muxer_merger_selection will not merge data and error information if
// the SLIDAS is in FEMB mode. This allows for data bits in control words
// when not acting as an LDC.
  
  wire [1:0] muxer_merger_selection = (selection3_[9:8] == 2'b11)?
                                             (femb?2'b10:2'b11):   // yes
				       	      selection3_[9:8];     // no


//======================================================================
//         Pipelines for lctrl, lwen, lderr and selection
//======================================================================

  lctrl_pipe lctrl_pipe1 (lclk, sysenable, pipe_reset, int_lctrl, lctrl_n);

  lwen_pipe lwen_pipe1 (lclk, sysenable, pipe_reset, int_lwen, lwen_n);
  
  lderr_pipe lderr_pipe1 (lclk, sysenable, pipe_reset, int_lderr, lderr_n);

  selection_pipe selection_pipe1 (lclk, sysenable, pipe_reset, selection,
                  selection2_, selection3_);
		       
//======================================================================
//         First level: the data sources 
//======================================================================

  walking_and_inverting walk_and_invert_gen(lclk, sysenable,
                  walking_and_inverting_reset, walking_and_inverting_enable,
                  pattern_switch, udw, walk_and_invert_q);

  random_data random_gen(lclk, sysenable, random_reset, random_enable,
                  random_q);

  look_up_table look_up_table_gen (lclk, sysenable, selection[3:0],
                  look_up_table_q, pattern0_jumper_on);

  block_counter block_counter_gen (lclk, sysenable, block_counter_reset,
                  block_counter_enable, 16'b0, block_counter_q);

  trigger_type trigger_type_gen (lclk, sysenable, trigger_type_reset,
                  trigger_type_enable, trigger_type_q);

  data_counter data_counter_gen (lclk, sysenable, data_counter_reset,
                  data_counter_enable, 16'b0, data_counter_q);

  error_gen error_gen_gen (lclk, sysenable, error_gen_reset, error_gen_enable,
                  inj_err, error_gen_start, int_lctrl, ctrl_err, data_err,
		  block_basis, int_lderr, error_gen_q, inj_err_led);

		  
//======================================================================
//        Second level: first multiplexer level
//======================================================================
		  
  big_multiplexer big_multiplexer0 (lclk, sysenable,
                  selection2_[7:6], walk_and_invert_q, random_q,
		  look_up_table_q, block_counter_q, big_muxer0_q);

  big_multiplexer big_multiplexer1 (lclk, sysenable,
                  selection2_[5:4], trigger_type_q, data_counter_q,
		  {28'bX, error_gen_q},
		  {16'b0, block_length}, big_muxer1_q);


		  
//======================================================================
//        Third level: second multiplexer level
//======================================================================
  muxer_merger mux_merger0 (lclk, sysenable, muxer_merger_selection,
		  big_muxer0_q, big_muxer1_q, mux_merger_q);


//======================================================================
//        Fourth level: output signals
//======================================================================
		  
  outff_32 ld_out (lclk, sysenable, udw, mux_merger_q, ld);
  
endmodule




//======================================================================//
//                          Module declarations                         //
//======================================================================//

/*
`ifdef exemplar


module lctrl_pipe (lclk, sysenable, pipe_reset, int_lctrl, lctrl_n);
  input lclk, sysenable, pipe_reset, int_lctrl;
  output lctrl_n;
endmodule


module lderr_pipe (lclk, sysenable, pipe_reset, int_lderr, lderr_n);
  input lclk, sysenable, pipe_reset, int_lderr;
  output lderr_n;
endmodule


module lwen_pipe (lclk, sysenable, pipe_reset, int_lwen, lwen_n);
  input lclk, sysenable, pipe_reset, int_lwen;
  output lwen_n;
endmodule
  
  
module selection_pipe (lclk, sysenable, pipe_reset, selection,
                       selection2_, selection3_);
  input lclk, sysenable, pipe_reset;
  input [9:0] selection;
  output [9:0] selection2_, selection3_;
endmodule

  
module walking_and_inverting (lclk, sysenable, reset, enable,
                  pattern_switch, udw, q);
  input lclk, sysenable, reset, enable;
  input [1:0] udw;
  input [3:0] pattern_switch;
  output [31:0] q;
endmodule


module random_data (lclk, sysenable, reset, enable, q);
  input  lclk, sysenable, reset, enable;
  output [31:0] q;
endmodule


module look_up_table2 (lclk, sysenable, index, q, pattern0_jumper_on, 
			block_length);
  input lclk, sysenable, pattern0_jumper_on;
  input [3:0] index;
  input [15:0] block_length;
  output [31:0] q;
endmodule


module block_counter (lclk, sysenable, reset, count_up, d, q);
  input lclk, sysenable, reset, count_up;
  input [15:0] d;
  output [31:0] q;
endmodule


module trigger_type (lclk, sysenable, reset, count_up, q);
  input lclk, sysenable, reset, count_up;
  output [31:0] q;
endmodule


module data_counter (lclk, sysenable, reset, count_up, d, q);
  input lclk, sysenable, reset, count_up;
  input [15:0] d;
  output [31:0] q;
endmodule


module error_gen (lclk, sysenable, reset, enable, inj_err, start,
                  int_ctrl, ctrl_err, data_err, block_basis, int_lderr,
		  ld[3:0], inj_err_led);
  input lclk, sysenable, reset, enable, inj_err, start, int_ctrl;
  input ctrl_err, data_err, block_basis;
  output int_lderr, inj_err_led;
  output [3:0] ld;
endmodule


module big_multiplexer (lclk, sysenable, sel, in0, in1, in2, in3, q);
  input lclk, sysenable;
  input [1:0] sel;
  input [31:0] in0, in1, in2, in3;
  output [31:0] q;
endmodule


module muxer_merger (lclk, sysenable, sel, in0, in1, q);
  input lclk, sysenable;
  input [1:0] sel;
  input [31:0] in0, in1;
  output [31:0] q;
endmodule


module outff_32 (lclk, sysenable, udw, d, q);
  input lclk, sysenable;
  input [1:0] udw;
  input [31:0] d;
  output [31:0] q;
endmodule


`else
*/

`include "lctrl_pipe.v"
`include "lderr_pipe.v"
`include "lwen_pipe.v"
`include "selection_pipe.v"
`include "walking_and_inverting.v"
`include "random_data.v"
`include "look_up_table4.v"
`include "block_counter.v"
`include "trigger_type.v"
`include "data_counter.v"
`include "error_gen.v"
`include "big_multiplexer.v"
`include "muxer_merger.v"
`include "outff_32.v"

//`endif
