/* 

   Project: Altera circuit of SLIDAS 

   File name: duty_controller.v
   Version:   4.2
   Updated:   30/8/99 

   Author: Gillian Damerell


   
-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
      4.2    |  GD      |  3/8/99     | This is based on the duty_controller.v
             |          | 	      | written by PBY.  The EVERY_X modes have
	     |		| 	      | been swapped for N kHz modes, and 
	     |		|	      | running is generally set by the output
	     |		|	      | from the trigger counters.
	     |		|	      |
-------------------------------------------------------------------------------



     Module description:
     -------------------

     This module gives the pace of the Slidas. For this it uses two signals:
     clock_enable and running, which work in different ways depending on
     the duty mode selected by SW2.

     the signals clock_enable and running
     ====================================
     clock_enable should in fact be called word_enable, because for generating
     a new word this signal must be high.
     
     During RUN and SINGLE BLOCK modes, clock_enable will be permanently high
     The difference between sending data continuously or by blocks is decided
     by the signal 'running': during RUN mode 'running' will toggle at each
     press of the RUN/STEP push button, while during SINGLE BLOCK mode
     'running' is set to trigger_out, which will be high whenever the external 
     trigger counter is above zero.  The state machine senses the state of 
     'running' only before starting a new block. This allows for blocks to be 
     sent out entirely before stopping.

     During SINGLE STEP mode, 'running' is constantly high and clock_enable
     is pulsed after each press on RUN/STEP.

     During the N kHz mode, 'running' is set to trigger_out, which will be 
     high whenever the internal trigger counter is above zero.  clock_enable 
     toggles when the RUN/STEP button is pressed, acting as an overall on/off 
     switch.
     
     (in the explanation above, pressing RUN/STEP can also mean a rising
     edge in the LEMO connector)

     
     
///////////////////////////////////////////////////////////////////////////////
*/


module duty_controller (lclk, sysreset, ureset, duty_switch, switch_step,
                        lemo_step, trigger_out, clock_enable, running);
		 
  input lclk, sysreset, ureset, switch_step, lemo_step, trigger_out;
  input [3:0] duty_switch;
  output clock_enable, running;

  
  reg clock_enable;	    // use of clock_enable and running is explained
  reg running;              // above.

  reg step;

  wire lemo_step_p;         // a synchronous pulse indicating a rising edge
                            // at the LEMO connector.  
  wire lemo_l;              // output of the flip-flop that is clocked by
                            // the rising edge at the LEMO connector input.  
  wire lemo_s;              // synchronised version of lemo_l

  
  parameter
    RUN          = 0,
    SINGLE_STEP  = 1,
    SINGLE_BLOCK = 2,
    _50kHz       = 3,
    _75kHz       = 4,
    _90kHz       = 5,
    _100kHz      = 6,
    _110kHz      = 7,
    _125kHz	 = 8,
    _150kHz	 = 9;




//-----------------------------------------------------------------------------
// Synchronisation of the pulse at the LEMO connector
//
  edge_latch lemo_latch  (lemo_l, lemo_step, lemo_s);  // edge_latch is reset
                                                       // by lemo_s.
  sync_level lemo_sync   (lclk, lemo_s, lemo_l);
  sync_pulse lemo_pulse  (lclk, lemo_step_p, lemo_s);
//-----------------------------------------------------------------------------
  


//-----------------------------------------------------------------------------
// Step is the OR of the synchronous pulses from the LEMO and the RUN/STEP
// push button.
//
  always @(posedge lclk) step <= #1 lemo_step_p | switch_step;
//-----------------------------------------------------------------------------


    
//-----------------------------------------------------------------------------
// Here clock_enable is set up

  always @(posedge lclk)
    if (sysreset | ureset) clock_enable <= #1 0;
    else  
      case(duty_switch)
	SINGLE_STEP       : clock_enable <= #1 step;
	RUN, SINGLE_BLOCK : clock_enable <= #1 1;
	default		  : if(step) clock_enable <= #1 ~clock_enable;
      endcase
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Here running is set up

  always @(posedge lclk)
    if (sysreset | ureset) running <= #1 0;
    else
      case(duty_switch)
	SINGLE_STEP : running <= #1 1;
	RUN	    : if(step) running <= #1 ~running;
	default     : running <= #1 trigger_out;
      endcase
//-----------------------------------------------------------------------------

endmodule      


/*
`ifdef exemplar

module sync_level(lclk, out, in);
  input lclk, in;
  output out;
endmodule

module sync_pulse(lclk, out, in);
  input lclk, in;
  output out;
endmodule

module edge_latch (q, edge_in, reset);
  input edge_in, reset;
  output q;
endmodule


`else

`include "sync_level.v"
`include "sync_pulse.v"
`include "edge_latch.v"

`endif
*/


