/*

   Project: Altera circuit of SLIDAS 

   File name: word_counter.v
   Version:   4.2
   Updated:   99/08/26 13:56:46

   Author: Ph. Brodier-Yourstone, ECP division, CERN


-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
     1.1     |   PBY    | 14-02-1997  |  - Original coding
             |          |             |
     1.2     |   PBY    | 25-02-1997  |  - No changes at all.
             |          |             |
     1.3     |   PBY    | 05-03-1997  |  - Added comment header
             |          |             |
     3.1     |   PBY    | 06-04-1997  |  - Changed word_count_en to become
             |          |             |  word_count_enable.     
             |          |             |  - Removed some intermediate variables,
	     |          |             |  which were not necessary (their
             |          |             |  purpose was to optimise timing, but
             |          |             |  they were finally not necessary).
             |          |             |  - end_of_block gets activated one
             |          |             |  clock cycle earlier, to compensate for
             |          |             |  the additional clock delay introduced
             |          |             |  in the state machine when using sub-
             |          |             |  modules.	     
	     |          |             |
-------------------------------------------------------------------------------



     Module description:
     -------------------

     This module controls the amount of data words being sent by knowing
     how many words it has to send and telling the state_machine when
     it has to stop telling data_gen to send data words.
     
     The amount of words to be sent is indicated by block_length, which
     is loaded by the counter (length_counter).

/////////////////////////////////////////////////////////////////////////////
*/


module word_counter (lclk, sysenable, clock_enable, word_count_reset,
                     word_count_enable, block_length, end_of_block);


//---------------------------------------------------------------------------
// Inputs and outputs
		     
  input lclk, sysenable;       
  input clock_enable;         // comes from duty_controller, where it is
                              // explained in detail.
  
  input [15:0] block_length;  // Tells how many words to count. It is
                              // generated by length_controller.
  
  input word_count_reset;     // These signals steer the counter. word_count_
                              // reset resets the counter by loading it with
  input word_count_enable;    // block_length. word_count_enable enables it.
 
  
  output end_of_block;        // tells the state machine that the words have
  reg end_of_block;           // been counted and the data block is finished.
                              // It is generated before getting to 0, to
                              // compensate for the clock cycles that are used
                              // in the following way:
                              //
                              // 1) The state machine issues word_count_reset
                              // at the same time as it issues the request for
                              // the first data word and not before, like
                              // during the control word. This provides
                              // flexibility in the sense that one doesn't need
                              // to have a particular word before the data
                              // words, nor a special state, in order to assert
                              // word_count_reset. So, with the first data word
                              // the counter gets the value 'block_length'.
                              // With this configuration, the counter should
                              // assert end_of_block when reaching the value
                              // of 1 (let's call this the Bottom value).
                              //
                              // 2) There is one clock cycle delay between the
                              // state machine and this module, so add 1 to the
                              // Bottom value.
                              //
                              // 3) The state machine reacts to end_of_block
                              // one clock cycle after receiving it.
                              // => Bottom = Bottom+1 = 3
                              //
                              // 4) The state machine wants to receive 
                              // end_of_block during the clock cycle when it is
                              // issuing the last data word request and not one
                              // clock cycle later, where it would choose 
                              // between new data word or control word or 
                              // whatever depending on end_of_block.
                              // => Bottom = Bottom+1 = 4
                              // 
                              // Conclusion: end_of_block must become 1 at the
                              // same time as the length_counter would reach 
                              // the value 4.
//-----------------------------------------------------------------------------


  
//-----------------------------------------------------------------------------
// Internal registers
  
  reg BL_is_4;                // Flag that is set to 1 if block_length is 4.
                              // It is used to set end_of_block directly as
                              // there is no time for using the counter
                              // 'length_counter'
  
  reg LC_is_5;                // Flag that is set to 1 when length_counter 
                              // reaches the value 5.
  
  reg [15:0] length_counter;  // The (by now) famous length_counter (see above)
//-----------------------------------------------------------------------------


  
//-----------------------------------------------------------------------------
// BL_is_4 is constantly updated
  
  always @(posedge lclk)
    BL_is_4 <= #1 (block_length == 4)?1:0;
//-----------------------------------------------------------------------------


    
//-----------------------------------------------------------------------------
// Here is generated the intermediate variable LC_is_5, which is used
// to assert end_of_block. 
    
  always @(posedge lclk)
    if (sysenable & clock_enable)            
      if (word_count_reset) begin     
	LC_is_5 <= #1 BL_is_4;            // this is not really useful...
      end
      else
      	if (word_count_enable) begin               // After the clock edge
	  LC_is_5 <= #1 (length_counter == 6)?1:0; // length_counter will 
      	end                                        // be 5.
//-----------------------------------------------------------------------------

	
	
//-----------------------------------------------------------------------------
// Below is generated end_of_block and length_counter is controlled.
// If the data block has a length of 4, end_of_block is set to 1 immediately
// at reset (we cannot use LC_is_5 here since it will never be set to 1 in
// this case, as the counter must pass by the value 6...)
//	
// For blocks of length 0 and 1 end_of_block is ignored by the state machine,
// so we don't look at that possibility here.
//
// For blocks larger than 4 words, end_of_block is asserted when length_counter
// reaches 4 (LC_is_5 is active while length_counter is 5, but since the
// assignment takes place after a clock cycle end_of_block will be asserted
// indeed when length_counter reaches 4).
	
  always @(posedge lclk)
    if (sysenable & clock_enable)
      if (word_count_reset) begin                // word_count_reset should
	length_counter <= #1 block_length;       // never be asserted more
	end_of_block <= #1 BL_is_4;              // than one clock_enable cycle
      end                                        // at a time in the state
      else                                       // machine. If it is asserted
      	if (word_count_enable) begin             // more than one clock cycle,
	 length_counter <= #1 length_counter - 1;// the 4-word blocks will
	  end_of_block <= #1 LC_is_5;            // become shorter because
      	end                                      // end_of_block will be set
    	else end_of_block <= #1 0;               // to 1 by BL_is_4 earlier
	                                         // than expected.
//-----------------------------------------------------------------------------

endmodule
