/*
/////////////////////////////////////////////////////////////////////////////
//
//   Project: Altera circuit of SLIDAS
//
//   File name: random_data.v
//   Version:   4.2
//   Updated:   99/08/26 13:56:32
//
//   Author: Ph. Brodier-Yourstone, ECP division, CERN
//
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////

     Module description:
     -------------------

     Pseudo random data generator. Implemented as a 32-bit shift register
     with four of its bits XORed and fed back to input of the shift register.


     The following picture shows which bits are tapped. The numbers show
     the bit numbers in the original shift register.

     In this module the bits have been rearranged to give an output that
     looks more random to the eye (otherwise the output be walking bits).
     However, the structure of the shift register and the XORed bits
     remains the same.

                                                         +-----+
       +------------------------------------------------>|     |
       |		   +---------------------------->|     |
       |		   |   +------------------------>| XOR |--+
       |		   |   |	       +-------->|     |  |
       |		   |   |	       |         +-----+  |
       |		   |   |	       |        	  |
     +---+---+-------+---+---+---+---+---+---+---+---+		  |
     |31 |30 |.......| 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |<-----------+
     +---+---+-------+---+---+---+---+---+---+---+---+
                      <--------------
                      shifts this way


		      
     Rearranging the bits
     ====================
     The following table shows the new position of the bits. On the left
     are the bit numbers of the picture's shift register. On the right
     are the position that they occupy in this module.

                        +----+----+
                        | 31 | 29 |-------------------+
                        | 30 | 13 |		      |
                        | 29 | 21 |		      |
                        | 28 | 17 |		      |
                        | 27 | 12 |		      |
                        | 26 | 27 |		      |
                        | 25 | 10 |		      |
                        | 24 | 23 |		      |
                        | 23 | 14 |		      |
                        | 22 | 26 |		      |
                        | 21 | 30 |		      |
                        | 20 | 24 |		      |
                        | 19 |  6 |		      |
                        | 18 | 20 |		      |
                        | 17 |  3 |		      |
                        | 16 | 22 |		      |
                        | 15 |  7 |		      |
                        | 14 |  9 |	 +----------+ |
                        | 13 |  2 |	 |	    | |
                        | 12 | 19 |      | +------+ | |
                        | 11 |  0 |      | |      | | |
                        | 10 |  1 |      | | +--+ | | |
                        |  9 |  8 |      | | |  | | | |
                        |  8 | 11 |      | | |  V V V V
                        |  7 | 15 |      | | | +-------+
                        |  6 | 31 |------+ | | |       |
                        |  5 |  5 |--------+ | |  XOR  |
                        |  4 |  4 |          | |       |
                        |  3 | 28 |          | +-------+
                        |  2 | 25 |          |     |
                        |  1 | 18 |----------+     |
                        |  0 | 16 |<---------------+
                        +----+----+



/////////////////////////////////////////////////////////////////////////////
*/



module random_data (lclk, sysenable, reset, enable, q);
  input  lclk, sysenable, reset, enable;
  output [31:0] q;
  reg    [31:0] q;

  always @(posedge lclk)
    if (sysenable) begin
      if (reset) q <= #1 32'h 524948e0;    // Start with a nice random value.
	                                   // This is actually not the first
	                                   // value to be seen on the random
	                                   // modes (SW1 = 5,6,7). The first
	                                   // value on those modes is the one
	                                   // obtained one clock cycle after
	                                   // reset (if enabled, of course).

      else if (enable) q <= #1 {q[5],  q[24], q[13], q[25],
	                        q[10], q[30], q[18], q[6],
	                        q[14], q[7],  q[17], q[3],
				q[0],  q[16], q[12], q[29]^q[31]^q[5]^q[18],
				q[31], q[26], q[21], q[27],
				q[15], q[23], q[2],  q[11],
				q[9],  q[20], q[4],  q[28],
				q[22], q[19], q[8],  q[1]};


    end
    else q <= #1 q;

endmodule

