/* 
   Project: Altera circuit of SLIDAS 

   File name: trigger_count_controller.v
   Version:   1.3
   Updated:   3/8/99 

   Author: Gillian Damerell, ECP division, CERN


   
-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
      1.2    |  GD      |  3/8/99     | This is a new module which controls the
             |          | 	      | trigger counter, and decides when it
	     |		| 	      | should operate.
	     |		|	      | 
     1.3     |	GD	|  9/8/99     | Now only need one counter, and reset is
	     |		|	      | better.
-------------------------------------------------------------------------------



     Module description:
     -------------------
The trigger counter does not operate at all when the speed_switch is set to 
positions 6 - 9, because then lclk will generally be going slower than the
trigger, which would cause all sorts of complications.  If the duty_switch is 
set to SINGLE_BLOCK mode, ext_trig_enable will be set to 1 and the counter will 
increment with pulses from the RUN/STOP button or the LEMO connector.  If the 
duty_switch is set to one of the N kHz modes, int_trig_enable will be set to 1, 
and the counter will increment with pulses from the internal trigger.
-----------------------------------------------------------------------------
*/

module trigger_count_controller(lclk, sysreset, ureset, internal_trigger, 
				switch_step, lemo_step, block_sent, 
				duty_switch, speed_switch, 
				trigger_out, busy, full, overflow);


  input lclk, sysreset, ureset, internal_trigger, switch_step, lemo_step, 
        block_sent;
  input [3:0] speed_switch, duty_switch;
  output trigger_out, busy, full, overflow;


  reg trig_disable, int_trig_enable, ext_trig_enable;
  reg step;
  reg trigger_in, trigger_reset;

  
  wire lemo_step_p;         // a synchronous pulse indicating a rising edge
                            // at the LEMO connector.  
  wire lemo_l;              // output of the flip-flop that is clocked by
                            // the rising edge at the LEMO connector input.
  wire lemo_s;              // synchronised version of lemo_l
 
  wire block_sent_p;         // a synchronous pulse indicating the arrival
                            // of a block_sent signal.  
  wire block_s;             // synchronised version of block_sent.

  wire sync_int_trig;        // a synchronous pulse indicating a rising edge
                            // of the internal trigger
  wire trig_l;              // output of the flip-flop that is clocked by
                            // the rising edge of the internal trigger.  
  wire trig_s;              // synchronised version of trig_l

 

  parameter
    RUN          = 0,
    SINGLE_STEP  = 1,
    SINGLE_BLOCK = 2;
   


//-----------------------------------------------------------------------------
// Synchronisation of the pulse at the LEMO connector
//
  edge_latch lemo_latch  (lemo_l, lemo_step, lemo_s);  // edge_latch is reset
                                                       // by lemo_s.
  sync_level lemo_sync   (lclk, lemo_s, lemo_l);
  sync_pulse lemo_pulse  (lclk, lemo_step_p, lemo_s);
//-----------------------------------------------------------------------------
  

//-----------------------------------------------------------------------------
// Step is the OR of the synchronous pulses from the LEMO and the RUN/STEP
// push button.
//
  always @(posedge lclk) step <= #1 lemo_step_p | switch_step;
//----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Synchronisation of the internal trigger to lclk instead of clk66.
//
  edge_latch trig_latch(trig_l, internal_trigger, trig_s);  // edge_latch is 
							    // reset by trig_s.
  sync_level trig_sync   (lclk, trig_s, trig_l);
  sync_pulse trig_pulse  (lclk, sync_int_trig, trig_s);
//-----------------------------------------------------------------------------
  


//-----------------------------------------------------------------------------
// Synchronisation of the block_sent signal.  (Which should already be
// synchronous, but needs to be turned into a pulse.)

  sync_level block_sync   (lclk, block_s, block_sent);
  sync_pulse block_pulse  (lclk, block_sent_p, block_s);
//-----------------------------------------------------------------------------
  

//-----------------------------------------------------------------------------
// Setting up when the trigger counter is to operate, and whether it is the
// internal or external trigger.

  always @(posedge lclk)  
    if (sysreset | ureset)
      begin
        trig_disable <= #1 1;
        int_trig_enable <= #1 0;
        ext_trig_enable <= #1 0;
      end

    else
      begin

        case (speed_switch)
          6,7,8,9  : trig_disable <= #1 1;   // Too slow to use triggers
          default  : trig_disable <= #1 0;
        endcase

        case (duty_switch)
          RUN, SINGLE_STEP, SINGLE_BLOCK  :  int_trig_enable <= #1 0;	
	  default                         :  int_trig_enable <= #1 1;
        endcase

	case (duty_switch)
	  SINGLE_BLOCK  :  ext_trig_enable <= #1 1;
	  default       :  ext_trig_enable <= #1 0;
	endcase

      end
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// The trigger which is input to the counter.

  always @(posedge lclk)
    if (int_trig_enable) trigger_in <= #1 sync_int_trig;
    else  
      begin
	if (ext_trig_enable) trigger_in <= #1 step;
	else trigger_in <= #1 0;
      end
//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
// The reset which is input to the counter.

  always @(posedge lclk)
    if (trig_disable) trigger_reset <= #1 1;
    else
      begin
	if (ext_trig_enable) trigger_reset <= #1 0;
	else
	  begin
	    if (int_trig_enable & step) trigger_reset <= #1 ~trigger_reset;
	    else trigger_reset <= #1 trigger_reset;
	  end
      end
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// The submodule which does the actual counting.

  trigger_counter count1(lclk, trigger_reset, trigger_in, block_sent_p, 
			 trigger_out, busy, full, overflow);

//-----------------------------------------------------------------------------


/*
//-----------------------------------------------------------------------------
  always @(posedge lclk)
    begin
      trigger_out <= #1 counter_out;
      busy        <= #1 counter_busy;
      overflow    <= #1 counter_overflow;
    end
//-----------------------------------------------------------------------------
*/
endmodule


/*
`ifdef exemplar

module sync_level(lclk, out, in);
  input lclk, in;
  output out;
endmodule

module sync_pulse(lclk, out, in);
  input lclk, in;
  output out;
endmodule

module edge_latch (q, edge_in, reset);
  input edge_in, reset;
  output q;
endmodule

module trigger_counter (lclk, trigger_reset, trigger_in, block_sent_p, 
 			counter_out, counter_busy, counter_overflow);
  input lclk, trigger_reset, trigger_in, block_sent_p;
  output counter_out, counter_busy, counter_overflow;
endmodule



`else
*/

//`include "sync_level.v"
//`include "sync_pulse.v"
//`include "edge_latch.v" 
`include "trigger_counter.v"

//`endif




