/* 
   Project: Altera circuit of SLIDAS 

   File name: int_trig_counter.v
   Version:   1.2
   Updated:   3/8/99 

   Author: Gillian Damerell, ECP division, CERN


   
-------------------------------------------------------------------------------
     SCCS    |          |             |
    Version  |  Author  |  Mod. Date  |          Changes made
-------------------------------------------------------------------------------
      1.2    |  GD      |  3/8/99     | This is a new module which acts a 
             |          | 	      | counter for the N kHz modes.
	     |		| 	      | 
	     |		|	      | 
	     |		|	      | 
	     |		|	      |
-------------------------------------------------------------------------------



     Module description:
     -------------------

When the trigger counter is to be used, trigger_reset will be set to 0 by the 
trigger_count_controller.v.  In that case, the counter will increase by 1 when 
it receives a trigger pulse, decrease by 1 when it receives a signal to say 
that a block has been sent, and remain the same if it receives both or neither 
of these signals.  The counter can only decrease if it has reached 1023, the 
maximum, and only increase if it is at 0.  counter_out is set to 1 whenever the 
count is greater than 0, counter_busy is set when the count is greater than 
511, and counter_overflow is set when the counter is at 1023.  When the trigger_
reset is set to 1 the counter is set to 0, and thus the three outputs are also 
set to 0.  
-----------------------------------------------------------------------------
*/


module trigger_counter (lclk, trigger_reset, trigger_in, block_sent_p, 
			trigger_out, busy, full, overflow);


  input lclk, trigger_reset, trigger_in, block_sent_p;
  output trigger_out, busy, full, overflow;

  reg trigger_out, busy, full, overflow;
  reg [10:0] counter;



//-----------------------------------------------------------------------------
// This looks very complicated, but it operates as described above

  always @(posedge lclk)  if(~trigger_reset)  // If no trigger_reset, counter
    begin				      // will operate.

      if (counter == 1023)		      // If counter is at maximum, 
	begin				      // only count down.
	  if (block_sent_p) counter<= #1 counter - 1;
	  else counter <= #1 counter;
	end

      else
	begin

	  if (counter == 0)		      // If counter is at 0, only
	    begin			      // count up.
	      if (trigger_in) counter <= #1 counter + 1;
	      else counter <= #1 counter;
	    end

	  else				    // Otherwise, count up or down
	    begin			    // depending on the input signals

	      if (trigger_in ~^ block_sent_p) counter <= #1 counter;
	      else
		begin
		  if (trigger_in) counter <= #1 counter + 1;
		  if (block_sent_p) counter <= #1 counter - 1;
		end
	    end
	end
    end
  else   counter <= #1 0;


//-----------------------------------------------------------------------------
// Here the outputs are set up.

  always @(posedge lclk)
    begin
      trigger_out  <= #1 (counter == 0)?0:1;
      busy         <= #1 (counter > 511)?1:0;
      full         <= #1 (counter == 1023)?1:0;
    end

  always @(posedge lclk)
    if(trigger_reset)  overflow <= #1 0;
    else 

      begin
	if (full)  overflow <= #1 1;
	else  overflow <= #1 overflow;
      end

endmodule

